import wx
import os

ICONS = {'.py':'source.png','.pdf':'pdf.png','.htm':'mozilla.png',
    '.html':'mozilla.png','.txt':'font_type1.png','.rtf':'font_type1.png',
    '.pyc':'compiled.png','pyd':'compiled.png','pyo':'compiled.png',
    '.png':'image.png','.jpg':'image.png','.tif':'image.png','.bmp':'image.png',
    '.gif':'image.png','.tiff':'image.png','.jpeg':'image.png','.svg':'image.png',}

class Browser (wx.TreeCtrl) :
    def __init__ (self, parent, id, init_path='', file_colors = {}, def_color=[]) :
        wx.TreeCtrl.__init__ (self, parent, id)
        #icons
        self.iconsListIndex = parent.parentPanel.iconsListIndex
        self.SetImageList(parent.parentPanel.iconsList)

        if (not init_path) or (not os.path.isdir(init_path)) :
            init_path = os.getcwd()

        self.init_path = init_path
        self.file_colors = file_colors

        if not def_color :
           self.def_color = wx.Colour (0,0,0)
        else :
            self.def_color = wx.Colour (def_color)

        self.SetBackgroundColour(wx.Colour(255,255,255))

        self.root = self.AddRoot( init_path )
        self.SetPyData(self.root, 0)
        self.SetItemImage(self.root,
            self.iconsListIndex['folder_tar.png'],
            which=wx.TreeItemIcon_Normal)
        self.SetItemImage(self.root,
            self.iconsListIndex['synaptic.png'],
            which       = wx.TreeItemIcon_Expanded)

        #self.update()

        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.OnExpand, self)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.OnCollapse, self)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivate, self)
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnClick)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnClick)

    def open(self, fname) : pass

    def OnCollapse ( self, event ) : pass
    def OnActivate ( self, event ) : pass

    def OnClick (self, event) :
        pt = event.GetPosition();
        item, flags = self.HitTest(pt)
        self.SelectItem(item)
        try :
            path = self.__getPath ( item )
        except : # invalid item
            return
        if os.path.isfile ( path ) : self.open( path )

    def __getPath (self, item) :
        path = self.GetItemText( item )
        parent = self.GetItemParent ( item )
        while parent != self.root :
           p = self.GetItemText( parent )
           if not p.endswith (os.sep) : p += os.sep
           path = p + path
           parent = self.GetItemParent ( parent )
        p = self.GetItemText( self.root )
        if not p.endswith (os.sep) : p += os.sep
        return p + path

    def OnExpand (self, event) :
        item = event.GetItem()
        if self.GetChildrenCount(item, 0) > 1 :
           return
        path = self.__getPath ( item )
        self.__fill ( item, path )

    def __getColorByFile (self, fname) :
        for f in self.file_colors :
            for e in f.split (';') :
               if fname.lower().endswith ( e.lower().strip() ) :
                 c = self.file_colors[f]
                 return wx.Colour( c[0], c[1], c[2] )
        return self.def_color

    def __file_cmp (self, a, b) : return cmp ( a.lower(), b.lower() )

    def __fill (self, item, path ) :
        if not path.endswith (os.sep) : path += os.sep
        dirs  = [d for d in os.listdir ( path ) if os.path.isdir(path+d)] # dirs
        files = [f for f in os.listdir ( path ) if os.path.isfile(path+f)] # files

        dirs.sort( self.__file_cmp )
        files.sort( self.__file_cmp )

        for d in dirs :
            child = self.AppendItem( item, d)
            self.SetPyData(child, 0)
            self.SetItemImage(child,
                self.iconsListIndex['folder.png'],
                which=wx.TreeItemIcon_Normal)
            self.SetItemImage(child,
                self.iconsListIndex['folder_open.png'],
                which       = wx.TreeItemIcon_Expanded)
            self.SetItemBold(child)
            subchild = self.AppendItem(child, '.')
            self.SetPyData(subchild, 0)

        for f in files :
            child = self.AppendItem( item, f)
            ext     = os.path.splitext(f)[-1].lower()
            if ext == '.py':
                self.SetItemBold(child)
                self.SetItemTextColour(item, self.__getColorByFile(f))
            if ICONS.has_key(ext):
                self.SetItemImage(child,
                    self.iconsListIndex[ICONS[ext   ]],
                    which=wx.TreeItemIcon_Normal)
            self.SetPyData(child, 0)
            self.SetItemTextColour( child, self.__getColorByFile(f))

    def update(self):
        self.CollapseAndReset(self.root)
        self.__fill(self.root, self.init_path)
        self.Expand(self.root)