#(c)www.stani.be (read __doc__ for more information)
import sm
INFO=sm.INFO.copy()

INFO['description']=\
"""Windows specific"""

__doc__=INFO['doc']%INFO
#_______________________________________________________________________________
import win32com.client as client
import win32com.client.gencache as gencache
import pythoncom, win32con, win32gui

def SetWaitCursor(wait):
    if wait:
        hCursor = win32gui.LoadCursor(0, win32con.IDC_WAIT)
    else:
        hCursor = win32gui.LoadCursor(0, 0)
    win32gui.SetCursor(hCursor)

def DispatchWithEvents(clsid, user_event_class, disp_class=None):
    """Hacked version by www.stani.be
    Use this first without disp_class and read in the console the module name
    and class. Rename to module name to human language (maybe with XP suffix)
    and import it directly. Pass disp_class with the given class.

    Original docstr:
    Create a COM object that can fire events to a user defined class.
    clsid -- The ProgID or CLSID of the object to create.
    user_event_class -- A Python class object that responds to the events.

    This requires makepy support for the COM object being created.  If
    this support does not exist it will be automatically generated by
    this function.  If the object does not support makepy, a TypeError
    exception will be raised.

    The result is a class instance that both represents the COM object
    and handles events from the COM object.

    It is important to note that the returned instance is not a direct
    instance of the user_event_class, but an instance of a temporary
    class object that derives from three classes:
    * The makepy generated class for the COM object
    * The makepy generated class for the COM events
    * The user_event_class as passed to this function.

    If this is not suitable, see the getevents function for an alternative
    technique of handling events.

    Object Lifetimes:  Whenever the object returned from this function is
    cleaned-up by Python, the events will be disconnected from
    the COM object.  This is almost always what should happen,
    but see the documentation for getevents() for more details.

    Example:

    >>> class IEEvents:
    ...    def OnVisible(self, visible):
    ...       print "Visible changed:", visible
    ...
    >>> ie = DispatchWithEvents("InternetExplorer.Application", IEEvents)
    >>> ie.Visible = 1
    Visible changed: 1
    >>>
    """
    # Create/Get the object.
    disp = client.Dispatch(clsid)
    if not disp_class:
        if not disp.__dict__.get("CLSID"): # Eeek - no makepy support - try and build it.
            try:
                ti = disp._oleobj_.GetTypeInfo()
                disp_clsid = ti.GetTypeAttr()[0]
                tlb, index = ti.GetContainingTypeLib()
                tla = tlb.GetLibAttr()
                gencache.EnsureModule(tla[0], tla[1], tla[3], tla[4], bValidateFile=0)
                # Get the class from the module.
                disp_class = gencache.GetClassForProgID(str(disp_clsid))
                print 'Next time use:'
                print disp_class.__class__
            except pythoncom.com_error:
                raise TypeError, "This COM object can not automate the makepy process - please run makepy manually for this object"
        else:
            disp_class = disp.__class__
    # If the clsid was an object, get the clsid
    clsid = disp_class.CLSID
    # Create a new class that derives from 3 classes - the dispatch class, the event sink class and the user class.
    import new
    events_class = client.getevents(clsid)
    if events_class is None:
        raise ValueError, "This COM object does not support events."
    result_class = new.classobj("COMEventClass", (disp_class, events_class, user_event_class), {"__setattr__" : client._event_setattr_})
    print dir(result_class)
    instance = result_class(disp._oleobj_) # This only calls the first base class __init__.
    events_class.__init__(instance, instance)
    if hasattr(user_event_class, "__init__"):
        user_event_class.__init__(instance)
    return client.EventsProxy(instance)

def Browse (ob,title='Python Object Browser'):
    " Browse the argument, or the main dictionary "
    import pywin.tools.browser as b
    class dynamic_browser(b.dynamic_browser):
        def __init__(self,hli_root,title='Python Object Browser'):
            self.dt[0][0]=title
            b.dynamic_browser.__init__(self,hli_root)
    root = b.MakeHLI (ob, 'root')
    if not root.IsExpandable():
        raise TypeError, "Browse() argument must have __dict__ attribute, or be a Browser supported type"
    dlg = dynamic_browser (root,title=title)
    dlg.CreateWindow()

class PythonWin:
    """PythonWin IDE as a Class"""
    def __init__(self,fileName=None,browse=[],browseMDI=[],modules='',fileList=[],onIdle=None):
        """Constructor
        BrowseMDIs: objects to be opened in a browser window
        fileList:   list of files to be preloaded
        """
        try:
            import os,sys
            import pywin.tools.browser,win32ui
            from pywin.framework.app import CreateDefaultGUI
            from pywin.framework.intpyapp import InteractivePythonApp
            #from pywin.tools.browser import Browse
        except ImportError,message:
            print 'ImportError: PythonWin (%s)'%message
            return
        class _PythonWinNoDDE(InteractivePythonApp):
            """No dde methods."""
            def InitDDE(self):
                pass
            def MakeExistingDDEConnection(self):
                pass
        stderr=sys.stderr
        stdout=sys.stdout
        self.fileList=fileList
        self.setStatusText=win32ui.SetStatusText
        sys.argv=[]

        CreateDefaultGUI(_PythonWinNoDDE)
        self.app=win32ui.GetApp()
        for b in browse:
            Browse(*b)
        for b in browseMDI:
            pywin.tools.browser.BrowseMDI(b)
        self.app.LoadUserModules(modules)
        fileList=self.getFileList()
        if fileName and fileName not in fileList:fileList.append(fileName)
        for f in fileList:self.app.OpenDocumentFile(f)
        self.editor=self.app.GetDocTemplateList()[1]
        self.interpretor=self.app.GetDocTemplateList()[0].GetDocumentList()[0]
        self.browser=pywin.tools.browser.template
        if onIdle:self.app.idleHandlers.append(onIdle)
        win32ui.StartDebuggerPump()
        sys.stderr=stderr
        sys.stdout=stdout

    def getFileList(self):
        """Can be overwritten."""
        return self.fileList

    def getDocumentList(self):
        """Returns a list of the currently opened documents."""
        return [x.GetPathName() for x in self.editor.GetDocumentList()]

    def onIdle(self):
        """Idle handler: To be overwritten."""
        pass

class PythonWinPreload(PythonWin):
    """PythonWin IDE which automatically opens file which were open during
    the last editing session."""
    def __init__(self,path,fileName=None,browse=[],browseMDI=[],modules='',statusText='Ready'):
        """path is the file with the file list."""
        self.path=path
        self.statusText=statusText
        PythonWin.__init__(self,fileName,browse=browse,browseMDI=browseMDI,modules=modules,onIdle=self.onIdle)

    def getFileList(self):
        """Loads file list from the path."""
        try:
            f=open(self.path,'r')
            fileList=eval(f.read())
            f.close()
            return fileList
        except:
            return []

    def onIdle(self,*arg,**keyw):
        """Save file list to the path."""
        newFileList=self.getDocumentList()
        if newFileList!=self.fileList:
            self.fileList=newFileList
            f=open(self.path,'w')
            f.write(str(self.fileList))
            f.close()
            self.setStatusText(self.statusText)